#ifndef _FROG_ELEMENT_PRIMITIVE_RECTANGLE_H__
#define _FROG_ELEMENT_PRIMITIVE_RECTANGLE_H__

#include "FROG_Element_Base.h"
#include "FROG_Primitives.h"

class FROG_Element_Primitive_Rectangle : public FROG_Element_Base_With_DetId {
public :	
	float PosX;    float PosY;    float PosZ;					
	float WidthX;  float WidthY;  float WidthZ;
	float LengthX; float LengthY; float LengthZ;	
	FROG_Objects*  frogObjects_;

	virtual bool isCompactible(){ return true; }

	static unsigned int sizeOf(){ return FROG_Element_Base_With_DetId::sizeOf() + 9*sizeof(float);}

	FROG_Element_Primitive_Rectangle(
	unsigned int detId,
	float posX,    float posY,    float posZ,
	float widthX,  float widthY,  float widthZ,
	float lengthX, float lengthY, float lengthZ) :
	FROG_Element_Base_With_DetId(C_PRIMITIVE_RECTANGLE,detId),
	PosX(posX),			PosY(posY),			PosZ(posZ),
	WidthX(widthX),		WidthY(widthY),		WidthZ(widthZ),
	LengthX(lengthX),	LengthY(lengthY),	LengthZ(lengthZ)	
	{    
		size_ = sizeOf();
	}

	FROG_Element_Primitive_Rectangle(FILE* pFile) : FROG_Element_Base_With_DetId(C_PRIMITIVE_RECTANGLE)
	{   
		size_ = sizeOf();
		fread(&detId_           ,sizeof(detId_)			 ,1,pFile);				
		fread(&PosX             ,sizeof(PosX)            ,1,pFile);
		fread(&PosY             ,sizeof(PosY)            ,1,pFile);
		fread(&PosZ             ,sizeof(PosZ)            ,1,pFile);
		fread(&WidthX           ,sizeof(WidthX)          ,1,pFile);
		fread(&WidthY           ,sizeof(WidthY)          ,1,pFile);
		fread(&WidthZ           ,sizeof(WidthZ)          ,1,pFile);
		fread(&LengthX          ,sizeof(LengthX)         ,1,pFile);
		fread(&LengthY          ,sizeof(LengthY)         ,1,pFile);
		fread(&LengthZ          ,sizeof(LengthZ)         ,1,pFile);
	}

	virtual void write () {
		size_ = sizeOf();
		data_ = new unsigned char[size_-6];
		data_ = FillBuffer(	data_, &detId_,				sizeof(detId_));
		data_ = FillBuffer( data_, &PosX,				sizeof(PosX));
		data_ = FillBuffer( data_, &PosY,				sizeof(PosY));
		data_ = FillBuffer( data_, &PosZ,				sizeof(PosZ));
		data_ = FillBuffer( data_, &WidthX,				sizeof(WidthX));
		data_ = FillBuffer( data_, &WidthY,				sizeof(WidthY));
		data_ = FillBuffer( data_, &WidthZ,				sizeof(WidthZ));
		data_ = FillBuffer( data_, &LengthX,			sizeof(LengthX));
		data_ = FillBuffer( data_, &LengthY,			sizeof(LengthY));
		data_ = FillBuffer( data_, &LengthZ,			sizeof(LengthZ));
		data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );		
	}

	#ifdef FROG_OPENGL
	virtual void display(bool UseDisplayList=true, float* color=NULL){
		// first Init Colors & Style!
		init(NULL);

		glLineWidth(style_->thickness_);
        if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}
		glLoadName( detId_ );

		
		
		glBegin (GL_QUADS); 	
			SetNormal(	PosX+WidthX +LengthX,PosY+WidthY +LengthY,PosZ+WidthZ +LengthZ,				// FIRST FACE
						PosX-WidthX +LengthX,PosY-WidthY +LengthY,PosZ-WidthZ +LengthZ,
						PosX-WidthX -LengthX,PosY-WidthY -LengthY,PosZ-WidthZ -LengthZ);

			glVertex3f (PosX+WidthX +LengthX,PosY+WidthY +LengthY,PosZ+WidthZ +LengthZ);	//+++	
			glVertex3f (PosX-WidthX +LengthX,PosY-WidthY +LengthY,PosZ-WidthZ +LengthZ);	//-++
			glVertex3f (PosX-WidthX -LengthX,PosY-WidthY -LengthY,PosZ-WidthZ -LengthZ);	//--+
			glVertex3f (PosX+WidthX -LengthX,PosY+WidthY -LengthY,PosZ+WidthZ -LengthZ);	//+-+



			SetNormal(	PosX+WidthX +LengthX,PosY+WidthY +LengthY,PosZ+WidthZ +LengthZ,				// SECOND FACE
						PosX+WidthX -LengthX,PosY+WidthY -LengthY,PosZ+WidthZ -LengthZ,
						PosX-WidthX -LengthX,PosY-WidthY -LengthY,PosZ-WidthZ -LengthZ);

			glVertex3f (PosX+WidthX +LengthX,PosY+WidthY +LengthY,PosZ+WidthZ +LengthZ);	//+++	
			glVertex3f (PosX+WidthX -LengthX,PosY+WidthY -LengthY,PosZ+WidthZ -LengthZ);	//+-+
			glVertex3f (PosX-WidthX -LengthX,PosY-WidthY -LengthY,PosZ-WidthZ -LengthZ);	//--+
			glVertex3f (PosX-WidthX +LengthX,PosY-WidthY +LengthY,PosZ-WidthZ +LengthZ);	//-++			
		glEnd();
	}
	#endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects*) frogObjects;			
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL && mother_->style_ !=NULL){		style_ = new FROG_Objects_Style(mother_->style_);
			}else{												style_ = new FROG_Objects_Style();	}	

			frogObjects_->frogCard_->GetColor( style_->color_     ,"Id_%i_Color"		,detId_);
			frogObjects_->frogCard_->GetFloat(&style_->thickness_ ,"Id_%i_Thickness"	,detId_);
		}
	}
};
#endif
