// FROG_Objects_Extended.h: interface for the FROG_Objects_Extended class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _FROG_OBJECTS_EXTENDED_H__
#define _FROG_OBJECTS_EXTENDED_H__

#include "FROG_Path.h"
#include "FROG_Objects.h"
#include "FROG_Element_Event.h"
#include "FROG_Events.h"
#include "FROG_Geometry.h"

//		glPointParameterfARB  = (PFNGLPOINTPARAMETERFARBPROC)wglGetProcAddress("glPointParameterfARB");
//		glPointParameterfvARB = (PFNGLPOINTPARAMETERFVARBPROC)wglGetProcAddress("glPointParameterfvARB");

//glPointParameterfARB = (int (*)(GLenum, const GLfloat)) dlsym(libHandle, "glPointParameterfARB");

//USE_GL_FUNC(PGLFNGLPOINTPARAMETERFARBPROC,       glPointParameterfARB);

#ifdef FROG_OPENGL
	#include "FROG_Texture.h"
#endif

class FROG_Objects_Extended : public FROG_Objects {
public : 
	FROG_Geometry*		 frogGeom_;
	FROG_Element_Event*  frogEvent_;
//	std::vector<unsigned int> EventToDisplay;
	FROG_Element_Base*   selectedObject_;
	bool				 selectedObject_DisplayFlag;

private : 
	std::map<unsigned int,unsigned int> Marker;


public :
	FROG_Objects_Extended() : FROG_Objects() {loadMarkers();selectedObject_=NULL;}

	FROG_Objects_Extended(FROG_ReadCards* frogCard) : FROG_Objects(frogCard){
	   loadMarkers();
	   selectedObject_=NULL;
	}

	FROG_Objects_Extended(FROG_Geometry* frogGeom, FROG_Element_Event* frogEvent, FROG_ReadCards* frogCard) : FROG_Objects(frogCard){
	   frogGeom_  = frogGeom;
	   frogEvent_ = frogEvent;
	   selectedObject_ = NULL;
	   loadMarkers();
	}

	FROG_Objects_Extended(FROG_Geometry* frogGeom, FROG_Element_Event* frogEvent, FROG_ReadCards* frogCard, FROG_Fonts* frogFonts) : FROG_Objects(frogCard, frogFonts){
	   frogGeom_  = frogGeom;
	   frogEvent_ = frogEvent;
	   selectedObject_ = NULL;
	   loadMarkers();
	}


   FROG_Geometry*      getGeometry() {	   return frogGeom_;   }
   FROG_Element_Event* getEvent()    {	   return frogEvent_;  }
   
   void setGeometry  (FROG_Geometry* geom)      {   frogGeom_ = geom;  }
   void setEvent     (FROG_Element_Event* Event){  frogEvent_ = Event; }

   unsigned int getMarker(int m){
	   if(m<=0  )return 0;
	   if(m>=200)return 0;
	   return Marker[m];   
   }

#ifdef FROG_OPENGL
   void loadMarkers(){
		char LocalPath[2048];
		char AbsolPath[2048];
		for(unsigned int i=1;i<200;i++){
		   sprintf(LocalPath,"Resources/Marker/%i.png",i);
		   if(FROG_PATH::CheckGlobalPath(LocalPath,frogCard_->mAbsolutePath.c_str(),AbsolPath)){
				unsigned int tmp = 0;				
				LoadTexture(&tmp,AbsolPath,true);
				Marker[i] = tmp;
		   }else{
				Marker[i] = 0;
		   }
		}
   }
#else
   void loadMarkers(){}
#endif

   std::vector<std::pair<bool,unsigned int> > GetHierarchy(FROG_Element_Base* This, unsigned int DetId=0){
		std::vector<std::pair<bool,unsigned int> > toReturn;
		FROG_Element_Base* tmp = NULL;

		if(DetId!=0){
			tmp = frogGeom_->FindByDetId(DetId);
			while(tmp){
				FROG_Element_Base_With_DetId* tmpId = dynamic_cast<FROG_Element_Base_With_DetId*>(tmp);
				if(tmpId) toReturn.push_back( std::make_pair(true,tmpId->DetId() ) );			
				tmp = tmp->getMother();
			}		
		}

		tmp = This;
		while(tmp){
			FROG_Element_Base_With_DetId* tmpId = dynamic_cast<FROG_Element_Base_With_DetId*>(tmp);			
			if(tmpId) toReturn.push_back( std::make_pair(false,tmpId->DetId() ) );
			tmp = tmp->getMother();
		}
		return toReturn;
   }

   bool IsSelected_Geometry(){
	   FROG_Element_Base* mother = selectedObject_;
	   while(mother!=NULL){
			if(mother->type()==C_GEOMETRY)return true;
			mother = mother->getMother();
	   }
	   return false;
   }
   
};


#endif

