/**************************************************************************
 **
 **   File:         ReadCards.cxx        
 **
 **   Description:  This is a *rubbish* parser. 
 **                 It ignores everything after a "//" until a "\n" and then 
 **                 ignores whitespace and \n using a getline stopping at
 **                 ";" and parsing the token name before the "=" and then
 **                 looking for "{" and "}". If finding them, doing a vector
 **                 if not, then use a single value.
 **
 **************************************************************************/ 

#include "FROG_ReadCards.h"

// repalce the std getline method,
// in order to be LINUX/MACOS/WINDOWS compatible
// Because those systems have different end line convention
istream& FROG_ReadCards::getLine(istream& is, string& buffer){
        char ch, peekch;
        char LF = '\n';
        char CR = '\r';
        is.get(ch);

//        string buffer;
		buffer.clear();

        while(!is.eof() && ch != LF && ch != CR){
                buffer.append(1, ch);
                is.get(ch);
        }
        
        if (is.eof()){
                return is;
        }
        
        peekch = is.peek();

		if(ch == CR && peekch == LF){ // CR/LF is windows otherwise Mac        
                is.get();
		}else if(peekch == CR){        // allow for improbable LF/CR        
                is.get();
        }

        return is;
}


// remove comments and whitespace
void FROG_ReadCards::clean() {

  ostringstream os(ostringstream::out);
  std::string line;

  while ( !mFile.eof() ) {    

	  getLine(mFile,line);
	//getline(mFile,line,'\n');
	//printf("%s\n",line.c_str());

	int pos = line.find("//");
    if ( pos != (int)std::string::npos ) {
      // check special cases : http:// and ftp://
      int pos2 = line.find("http://");
      int pos3 = line.find("ftp://");
      if( pos2  != pos-5  && pos3  != pos-4 ) {
        line = chop(line,"//");
      }
    }
 
    removespace(line);
    
    if ( line.size() )    os << line;
  }

  mString = os.str();
}


/**  parse the remaining cleaned input string
 **/

void FROG_ReadCards::parse()
{
  std::string line;
  std::string sline;
  
  while ( mString.size() ) {
    line  = chop(mString,";");
    sline = line;

    std::string         tagname;
    std::vector<string> values;
    
    // check if the tagname (before =) is not null
    if ( (tagname=chop(line,"=")).size()==0 ) error(sline);

    // check if the value (after =) has no "=" character
    if ( line.find("=") != std::string::npos ) error(sline);


    //  now parse the value

    std::string val;

	size_t pos = line.find("{");
    if ( pos != std::string::npos ) {
      // need to chomp off the braces if a vector
      if ( chop(line,"{").size() )  error(sline);
      if ( chomp(line,"}").size() ) error(sline); 
    }
    
    do {
      if ( (val=chop(line,",")).size()==0 ) error(sline);
      values.push_back(val);
    }
    while ( line.size() );

	if(strcmp(tagname.c_str(),"Include")==0 || strcmp(tagname.c_str(),"include")==0 || strcmp(tagname.c_str(),"#Include")==0 || strcmp(tagname.c_str(),"#include")==0){
		for(unsigned int z=0;z<values.size();z++){

			// check that the file is not already in the list
			bool alreadyIn = false;
		    char* FilePath = new char[2048];
			FROG_PATH::CheckGlobalPath(values[z].c_str(),mAbsolutePath.c_str(),FilePath);	  		
			for(unsigned int a=0;a<mFilesToRead.size()&&!alreadyIn;a++){
				if(strcmp(mFilesToRead[a],FilePath)==0)
					alreadyIn=true;
			}
			if(!alreadyIn){
			   mFilesToRead.push_back(FilePath);			
			}

		}
	}else{
		mValues[tagname] = values;
	}
    
  }
 
}

void FROG_ReadCards::ReadParam() 
{
  clean();
  parse();
}

// ---------------------------------------

void FROG_ReadCards::Construct(const char* filename) {

	  char* FilePath = new char[2048];
	  FROG_PATH::CheckGlobalPath(filename,mAbsolutePath.c_str(),FilePath);	  		
	  mFilesToRead.push_back(FilePath);

      for(unsigned int i = 0;i<mFilesToRead.size();i++){
		printf("Loading Config File : %s\n",mFilesToRead[i]);	
		//printf("-----------------------------------------------------------------------\n");

		FILE* fp;		
		if ( (fp=fopen(mFilesToRead[i],"r"))==NULL ) { 
		  printf("ReadCards::Construct() file %s not found\n",mFilesToRead[i]); 
		  exit(0);
		}
		fclose(fp);

		mFile.clear();
		mFile.open(mFilesToRead[i]);
		mFile.seekg(0, ios::beg);

		ReadParam();

		mFile.close();
		//printf("-----------------------------------------------------------------------\n\n\n");
	}
  }
 
std::string FROG_ReadCards::chop(std::string& s1, std::string s2) {
    std::string::size_type pos = s1.find(s2);
    std::string s3;
    if ( pos == std::string::npos ) {
      s3 = s1;
      s1.erase(0, s1.size());
    }
    else {
      s3 = s1.substr(0, pos); 
      s1.erase(0, pos+s2.size());
    }
    return s3;
} 

std::string FROG_ReadCards::chomp(std::string& s1, std::string s2) {
    std::string::size_type pos = s1.find(s2);
    std::string s3;
    if ( pos == std::string::npos ) {
      s3 = s1;
      s1.erase(0,s1.size());    
    }
    else {
      s3 = s1.substr(pos+s2.size(),s1.size());
      s1.erase(pos,s1.size()); 
    } 
    return s3;
} 


  // remove whitespace
void FROG_ReadCards::removespace(std::string& s1) {
    std::string::size_type pos;
    while ( (pos = s1.find(" "))!=std::string::npos ) {
      s1.erase(pos, 1);
    }
/*    while ( (pos = s1.find("	"))!=std::string::npos ) {
      s1.erase(pos, 1);
    }
*/
} 
  
void FROG_ReadCards::error(const std::string s) {
    cerr << "ReadCards() syntax error: " << s << endl;

	size_t pos = s.find("\r");	
    if ( pos != std::string::npos ) {
		for(unsigned int i=1;i<pos;i++){
			printf(" ");
		}
		printf("^\n");
		printf("\nWe have detected that your file is using '\r' Carriage Return instead of standard '\n'\n");
		printf("This means your file is not in the unix format... please correct it!");
	}

	pos = s.find(";");	
    if ( pos == std::string::npos ){
		printf("\nDid you forget the ';' at the end of the line ?\n");
	}

    exit(0);
}

template< class T >
double FROG_ReadCards::handle(const std::string& s, std::string format) const {
    T d;
    char   temps[1024] = "duff"; 
    int    r=0;
    format+="%s";
    if ( (r=sscanf(s.c_str(), format.c_str(), &d, temps))==0 ) {
      cout << "ReadCards() error in input file string " << s << endl;
      exit(0);
    }
    if ( r!=1 || string(temps)!="duff" ) {
      cout << "ReadCards() error in input file string " << s << " r = "<< r << endl;
      exit(0);
    }
    //    cout << "r=" << r << "   d=" << d <<"  temps=" << temps << endl;
    return d;
}


// ---------------------------------------

void FROG_ReadCards::Dump(FILE* file){
	for(Values_Iterator it=mValues.begin();it!=mValues.end();it++){
		fprintf(file,"%s\n", it->first.c_str() );
		for(unsigned int i=0;i<it->second.size();i++){
			fprintf(file," --> %s\n", it->second[i].c_str() );
		}fprintf(file,"\n");
	}
}

bool FROG_ReadCards::IsTagPresent(const char* format, ...){
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return false;
	return true;  
}

std::vector<double> FROG_ReadCards::GetFloatVector(std::string tag) { 
    std::vector<double> dValues;
    std::vector< std::string > vecVals = mValues[tag];
    for ( unsigned int j=0 ;  j< vecVals.size() ; j++ )
	dValues.push_back( handle<double>(vecVals[j], "%lf") );
    return dValues;
}

double FROG_ReadCards::GetFloat(const char* tag) {
    return handle<double>( mValues[tag].front(), "%lf" );
}

/*
double FROG_ReadCards::GetFloat(const char* tag, double DefaultValue){
	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return DefaultValue;	
	return handle<double>( it->second.front(), "%lf" );	
}
*/

/*
double FROG_ReadCards::GetFloat(double DefaultValue, const char* format, ...){
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return DefaultValue;	
	return handle<double>( it->second.front(), "%lf" );	
}
*/

void FROG_ReadCards::GetFloat(float* Float, const char* format, ...)
{
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return;	
	*Float = (float) handle<double>( it->second.front(), "%lf" );	
}

void FROG_ReadCards::GetColor(float* colorV, const char* format, ...)
{
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return;

	if(it->second.size()<3 || it->second.size()>4)
		printf("Color Vector %s has not enough components : %i instead of 3 or 4\n",tag,it->second.size() );

    for(unsigned int j=0;j< it->second.size();j++)
	colorV[j] = (float) handle<double>(it->second[j], "%lf");
}


double FROG_ReadCards::GetInt(const char* tag) {
    return handle<int>( mValues[tag].front(), "%d" );
}

void FROG_ReadCards::GetInt(int* Int, const char* format, ...){
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return;	
	*Int = (int) handle<int>( it->second.front(), "%d" );	
}

void FROG_ReadCards::GetShort(short* Short, const char* format, ...){
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return;	
	*Short = (short) handle<int>( it->second.front(), "%d" );	
}

void FROG_ReadCards::GetUInt(unsigned int* Int, const char* format, ...){
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return;	
	*Int = (int) handle<int>( it->second.front(), "%u" );	
}

std::vector<int> FROG_ReadCards::GetIntVector(const char* format, ...){
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

    std::vector<int> toReturn;

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return toReturn;	
	for(unsigned int i=0;i<it->second.size();i++){
		toReturn.push_back((int) handle<int>( it->second[i],"%d" ) );
	}
	return toReturn;
}

std::vector<unsigned int> FROG_ReadCards::GetUIntVector(const char* format, ...){
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

    std::vector<unsigned int> toReturn;

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return toReturn;	
	for(unsigned int i=0;i<it->second.size();i++){
		toReturn.push_back((int) handle<int>( it->second[i],"%u" ) );
	}
	return toReturn;
}

void FROG_ReadCards::GetChar(char* Char, unsigned int sizeMax, const char* format, ...){
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return;	
	
	if(sizeMax>=it->second.front().size()){
		strcpy(Char,it->second.front().c_str());
	}else{
		printf("FROG_ReadCards::GetChar : Destination Char is too small to content the full Source char\n");
	}
}

double FROG_ReadCards::GetHex(const char* tag) {
    return handle<int>( mValues[tag].front(), "%x" );
}


std::string FROG_ReadCards::GetString(const char* format, ...){
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end())return string();	

    return mValues[tag].front();
}

bool FROG_ReadCards::GetBool(bool Default, const char* format, ...){
	char tag[255];
	va_list args;
	va_start(args,format);
	vsprintf(tag,format,args);
	va_end(args);

	Values_Iterator it = mValues.find(tag);
	if(it==mValues.end()){if(strcmp(tag,"ViewMenu_ShowId")==0)printf("NOTFOUND\n");return Default;	}
	
	if(strcmp(it->second.front().c_str(),"true")==0)return true;
	return false;
}

std::vector<string> FROG_ReadCards::GetStringVector(const char* tag){
    return mValues[tag];
}
