

#include "FROG_Texture.h"
#include <iostream>

stTexture* VerticalFlipTexture(stTexture* Source)
{
	stTexture* toReturn      = new stTexture;

	toReturn->width          = Source->width ;
	toReturn->height         = Source->height;
	toReturn->format         = Source->format;
	toReturn->internalFormat = Source->internalFormat;	
	toReturn->texels         = new unsigned char[toReturn->width*toReturn->height*toReturn->internalFormat];

	for(unsigned int x=0;x<Source->width;x++){ 
		for(unsigned int y=0;y<Source->height;y++){
			unsigned int i = (x + y * Source->width  )*Source->internalFormat;

			unsigned int y2 = Source->height-1-y;
			unsigned int j = (x + y2 * toReturn->width)*toReturn->internalFormat;

			for(int c=0;c<toReturn->internalFormat;c++){
				toReturn->texels[j+c] = Source->texels[i+c];
			}
		}
	}

	delete [] Source->texels;
	delete Source;

	return toReturn;
}

void LoadTexture(unsigned int* id, const char* strFileName, bool VerticalFlip)
{	
	if(!strFileName) return;

	char		FileFormat[5];	
	strncpy(FileFormat, strFileName + strlen(strFileName)-4, 4);FileFormat[4]='\0';

	stTexture*	Texture = new stTexture;
	if(!strcmp(FileFormat, ".PNG") || !strcmp(FileFormat, ".png")){
		LoadPNG(Texture,strFileName);
	}else{		
	}

	if(Texture->texels == NULL){
		delete Texture;
		return;
	}

	if(VerticalFlip){
		Texture = VerticalFlipTexture(Texture);
	}

	glGenTextures (1, id);
	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
	glBindTexture (GL_TEXTURE_2D, *id);

	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	gluBuild2DMipmaps(GL_TEXTURE_2D, Texture->internalFormat, Texture->width, Texture->height, Texture->format, GL_UNSIGNED_BYTE, Texture->texels);

	delete [] Texture->texels;
	delete Texture;
}

void SaveScreenshot(const char* strFileName,unsigned int Width, unsigned int Height, void (*func)(void))
{		
	if(!strFileName) return;
		
	short		ExtSize=0;
	for(unsigned int i=strlen(strFileName);i>=0&&ExtSize==0;i--){if(strFileName[i]=='.')ExtSize=strlen(strFileName)-i-1;}
	char*		FileFormat = new char[ExtSize+2];

	strncpy(FileFormat, strFileName + strlen(strFileName)-(ExtSize+1), (ExtSize+1));FileFormat[ExtSize+1]='\0';	
	
	//To LowerCase
	for(short i=0;i<ExtSize+1;i++){	
		if(FileFormat[i]>64&&FileFormat[i]<90)FileFormat[i]+=32;
	}

	printf("Making a ScreenShot with Name %s... ",strFileName);
	if(!strcmp(FileFormat, ".png")){
			stTexture* Texture      = new stTexture;
			Texture->width          = Width;
			Texture->height         = Height;
			Texture->format         = GL_RGB;
			Texture->internalFormat = 3;
			Texture->texels         = new GLubyte[Texture->width*Texture->height*Texture->internalFormat];
			glReadPixels(0, 0, Texture->width, Texture->height, Texture->format, GL_UNSIGNED_BYTE, Texture->texels);
			SavePNG(Texture,strFileName);
			delete [] Texture->texels;
			delete Texture;
	}else if(!strcmp(FileFormat, ".ps" ) || !strcmp(FileFormat, ".eps") || !strcmp(FileFormat, ".tex") ||
		     !strcmp(FileFormat, ".pdf") || !strcmp(FileFormat, ".svg") || !strcmp(FileFormat, ".pgf")){

			FILE* fp = fopen(strFileName, "wb");			
			int state = GL2PS_OVERFLOW;
			int buffsize = 0;
			while(state == GL2PS_OVERFLOW){			  
			  buffsize += 2*4096*4096;
			  GLint Format = 0;

			  if(!strcmp(FileFormat, ".ps" ))Format|=GL2PS_PS;
			  if(!strcmp(FileFormat, ".eps"))Format|=GL2PS_EPS;
			  if(!strcmp(FileFormat, ".tex"))Format|=GL2PS_TEX;
			  if(!strcmp(FileFormat, ".pdf"))Format|=GL2PS_PDF;
			  if(!strcmp(FileFormat, ".svg"))Format|=GL2PS_SVG;
			  if(!strcmp(FileFormat, ".pgf"))Format|=GL2PS_PGF;

 			  GLint viewport[4]; 	
 			  glGetIntegerv(GL_VIEWPORT, viewport);
			  
			  gl2psEnable(GL2PS_BLEND);
			  gl2psEnable(GL2PS_LINE_STIPPLE);			  
			  gl2psBeginPage("FROG_ScreenShot", "FROG", viewport, Format, GL2PS_SIMPLE_SORT, 
//							 GL2PS_LANDSCAPE | GL2PS_OCCLUSION_CULL, 
							 GL2PS_DRAW_BACKGROUND | 
							 GL2PS_OCCLUSION_CULL, 							 
							 GL_RGBA, 0, NULL, 0, 0, 0, buffsize, fp, strFileName);
			  func();
			  state = gl2psEndPage();
			}
			fclose(fp);
	}else{		
		printf("Unknown Format : %s\n",FileFormat);return;
	}

	printf("Done!\n");
}


void LoadPNG(stTexture* Texture, const char* strFileName)
{
	png_structp png_ptr;
	png_infop   info_ptr;
	int number_of_passes;	
	int color_type;
	int bit_depth;

	unsigned char header[8];	// 8 is the maximum size that can be checked

	//open file and test for it being a png
	FILE *fp = fopen(strFileName, "rb");
	if (!fp) return;

	fread(header, 1, 8, fp);	
	if (png_sig_cmp(header, 0, 8)) return;

	//initialize stuff
	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	
	if (!png_ptr) return;		

	info_ptr = png_create_info_struct(png_ptr);
	if (!info_ptr) return;
		
	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, 8);
	png_read_info(png_ptr, info_ptr);

	Texture->width  = info_ptr->width;
	Texture->height = info_ptr->height;
	color_type      = info_ptr->color_type;
	bit_depth       = info_ptr->bit_depth;

	if(color_type == PNG_COLOR_TYPE_PALETTE) {
		png_set_palette_to_rgb(png_ptr); }							// on remplace la palette par du rgb
	if(color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8) {
		png_set_gray_1_2_4_to_8 (png_ptr);	}						// on converti au format 8bit par couleur	
	if (png_get_valid (png_ptr, info_ptr, PNG_INFO_tRNS)){
		png_set_tRNS_to_alpha (png_ptr);	}


	if (bit_depth == 16){
		png_set_strip_16 (png_ptr);}								// si le canal couleur est sur 16bits, on le ramene  8bits	
	else if (bit_depth < 8){
		png_set_packing (png_ptr);}									// si le canal couleur est inferieur a 8bits on le ramene  8bits
	
	number_of_passes = png_set_interlace_handling(png_ptr);	
	png_read_update_info(png_ptr, info_ptr);


	switch (color_type)
	{
		case PNG_COLOR_TYPE_GRAY:
		    Texture->format			= GL_LUMINANCE;
			Texture->internalFormat	= 1;
		break;

		case PNG_COLOR_TYPE_GRAY_ALPHA:
			Texture->format			= GL_LUMINANCE_ALPHA;
			Texture->internalFormat = 2;
		break;

		case PNG_COLOR_TYPE_RGB:
			Texture->format			= GL_RGB;
			Texture->internalFormat = 3;
		break;

		case PNG_COLOR_TYPE_RGB_ALPHA:
		    Texture->format			= GL_RGBA;
			Texture->internalFormat = 4;
		break;

		default:
			// Badness 
		break;
	}
 
	//read file
	Texture->texels = new unsigned char [Texture->width * Texture->height * Texture->internalFormat];
	png_bytep * row_pointers = new png_bytep [Texture->height];
	for (unsigned int y=0; y<Texture->height; y++){
	    row_pointers[y] = (png_bytep)(Texture->texels +
						  ((Texture->height - (y + 1)) * Texture->width * Texture->internalFormat));
	}
	png_read_image(png_ptr, row_pointers);

	delete row_pointers;
	png_read_end(png_ptr,NULL);
	png_destroy_read_struct (&png_ptr, &info_ptr,NULL);

    fclose(fp);
}


void SavePNG(stTexture* Texture, const char* strFileName)
{
	if(Texture->texels==NULL || Texture->width==0 || Texture->height==0) return;

	png_structp png_ptr;
	png_infop   info_ptr;
	int color_type;
	int bit_depth;

	//create file
	FILE *fp = fopen(strFileName, "wb");
	if (!fp) return;	

	//initialize stuff
	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);	
	if (!png_ptr) return;

	info_ptr = png_create_info_struct(png_ptr);
	if (!info_ptr) return;

	png_init_io(png_ptr, fp);	
	switch (Texture->format)
	{
		case GL_LUMINANCE:	     color_type	= PNG_COLOR_TYPE_GRAY;		 break;
		case GL_LUMINANCE_ALPHA: color_type	= PNG_COLOR_TYPE_GRAY_ALPHA; break;
		case GL_RGB:			 color_type	= PNG_COLOR_TYPE_RGB;		 break;
		case GL_RGBA:		     color_type	= PNG_COLOR_TYPE_RGB_ALPHA;	 break;
		default:			     return;								 break;
	}	
	bit_depth  = 8;

	//write header	
	png_set_IHDR(png_ptr, info_ptr, Texture->width, Texture->height,
		     bit_depth, color_type, PNG_INTERLACE_NONE,
		     PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
	png_write_info(png_ptr, info_ptr);

	//write bytes
	png_bytep * row_pointers = new png_bytep[Texture->height];
	for (unsigned int y=0; y<Texture->height; y++){
	    row_pointers[y] = (png_bytep)(Texture->texels +
						  ((Texture->height - (y + 1)) * Texture->width * Texture->internalFormat));
	}
	png_write_image(png_ptr, row_pointers);
	
	//end write	
	png_write_end(png_ptr, NULL);

	delete [] row_pointers;
    fclose(fp);
}



