// FROG_View_Tools.h: interface for the FROG_View class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _FROG_VIEW_TOOLS_H__
#define _FROG_VIEW_TOOLS_H__

#include "FROG_Objects_Extended.h"

#include "FROG_View_Screen.h"
#include "FROG_View_2D.h"
#include "FROG_View_3D.h"
#include "FROG_View_Menu.h"

#include "FROG_View_LegoPlot_CaloHit.h"
#include "FROG_View_LegoPlot_CaloTower.h"
#include "FROG_View_LegoPlot_CaloJet.h"

#define VIEWSCREEN			1
#define VIEW2D				2
#define VIEW3D				3
#define VIEWLEGO			10
#define VIEWLEGO_CALOHIT	11
#define VIEWLEGO_CALOTOWER	12
#define VIEWLEGO_CALOJET	13
#define VIEWMENU			99


namespace FROG_VIEW{

FROG_View* Create(char* Name, FROG_Objects_Extended* frogObjects){
	char Type[255] = {"None"};

	if(frogObjects!=NULL){
		frogObjects->frogCard_->GetChar (Type,255, "%s_Type", Name);
	}else{
		strcpy(Type,Name);
	}
	printf("Creation of a New view of Type : \"%5s\" with Name %s\n",Type, Name);

	if(strcmp(Type,"Screen")==0){
		return new FROG_View_Screen("Screen", frogObjects);
	}else if(strcmp(Type,"3D")==0){		
		return new FROG_View_3D(Name, frogObjects);						
	}else if(strcmp(Type,"2D")==0){
		return new FROG_View_2D(Name, frogObjects);
	}else if(strcmp(Type,"LegoPlot_CaloHit")==0){
		return new FROG_View_LegoPlot_CaloHit(Name, frogObjects);
	}else if(strcmp(Type,"LegoPlot_CaloTower")==0){
		return new FROG_View_LegoPlot_CaloTower(Name, frogObjects);
	}else if(strcmp(Type,"LegoPlot_CaloJet")==0){
		return new FROG_View_LegoPlot_CaloJet(Name, frogObjects);
	}else if(strcmp(Type,"Menu")==0){
		return new FROG_View_Menu(Name, frogObjects);
	}else{
		printf("The View type \"%s\" does not exit\n",Type);
		printf("Frog will stop\n");
		exit(0);
	}

	return NULL;
}

void Swap(FROG_View* v1, FROG_View* v2 ){
//	printf("Swap : %s <--> %s\n",v1->Name_, v2->Name_);

	if( !(v1->isSwappable && v2->isSwappable) )return;

	float temp_card_vx_ = v1->card_vx_;
	float temp_card_vy_ = v1->card_vy_;
	float temp_card_vw_ = v1->card_vw_;
	float temp_card_vh_ = v1->card_vh_;

	v1->card_vx_        = v2->card_vx_;
	v1->card_vy_        = v2->card_vy_;
	v1->card_vw_        = v2->card_vw_;
	v1->card_vh_        = v2->card_vh_;

	v2->card_vx_        = temp_card_vx_;
	v2->card_vy_        = temp_card_vy_;
	v2->card_vw_        = temp_card_vw_;
	v2->card_vh_        = temp_card_vh_;	
	v1->Reshape(0,0);
	v2->Reshape(0,0);	
}

} // End of Namespace

#endif


