
#include <stdio.h>
#include "FROG_ZLib.h"

bool copy(const char* Input, const char* Output){
	FILE* src_stream = fopen(Input,"rb");
	if(!src_stream)return false;

	FILE* dst_stream = fopen(Output,"wb");
	if(!dst_stream){fclose(src_stream);return false;}

	unsigned char buffer[1024];
	unsigned int Read;
	while(feof(src_stream)==0){
		Read = fread(buffer,1,1024,src_stream);
		if(fwrite(buffer,1,1024,dst_stream)!=Read){printf("There is an error in the copy of %s to %s\n",Input,Output);return false;}
	}

	fclose(src_stream);
	fclose(dst_stream);

	return true;
}

bool FROG_ZLIB_IsGZip(char* fileName){
	unsigned int N = strlen(fileName);
	
	if(  fileName[N-3]!='.')					   return false;
	if(!(fileName[N-2]=='g' || fileName[N-2]=='G'))return false;
	if(!(fileName[N-1]=='z' || fileName[N-1]=='Z'))return false;
	return true;
}

bool FROG_ZLIB_IsUnZippedFileExist(char* source_path){
	unsigned int N = strlen(source_path);

	char* dest_path = new char[N];	

	if(FROG_ZLIB_IsGZip(source_path)){
		strncpy(dest_path,source_path,N-3);
		dest_path[N-3] = '\0';
	}else{
		strcpy(dest_path,source_path);
	}

	// Check the dest file do not already exist!
	FILE* dest_stream = fopen(dest_path,"r");
	if(dest_stream){		
		fclose(dest_stream);
		delete dest_path;
		return true;
	}

	delete dest_path;
	return false;
}


bool FROG_ZLIB_Deflate(char* source_path, bool keepIntermediateFile){
	unsigned int N = strlen(source_path);

	printf("Start Deflation of %s\n",source_path);

	char* dest_path = new char[N+3];	
	sprintf(dest_path,"%s.gz",source_path);
	
	FILE* source_stream = fopen(source_path,"rb");
	if(!source_stream){return false;}

	FILE* dest_stream   = fopen(dest_path,"wb");
	if(!dest_stream){return false;}

	int returnedValue =  FROG_ZLIB_Deflate(source_stream, dest_stream, 9);

	fclose(source_stream);
	fclose(dest_stream);

	if(returnedValue==Z_OK){
		printf("%s has been deflate corretly\n",source_path);				
		printf("new name is %s\n",dest_path);				
		if(!keepIntermediateFile && remove(source_path)!=0)perror( "Error deleting file" );
		return true;
	}else if(returnedValue==Z_MEM_ERROR){
		printf("### ERROR ### Not Enough Memory to deflate the file : %s\n",source_path);		
	}else if(returnedValue==Z_STREAM_ERROR){
		printf("### ERROR ### Stream error during the deflation of the file : %s\n",source_path);		
	}else if(returnedValue==Z_VERSION_ERROR){
		printf("### ERROR ### Version error during the deflation of the file : %s\n",source_path);		
	}else if(returnedValue==Z_ERRNO){
		printf("### ERROR ### Reading/Writing error during the deflation of the file : %s\n",source_path);		
	}	

	return false;
}


/* Compress from file source to file dest until EOF on source.
   def() returns Z_OK on success, Z_MEM_ERROR if memory could not be
   allocated for processing, Z_STREAM_ERROR if an invalid compression
   level is supplied, Z_VERSION_ERROR if the version of zlib.h and the
   version of the library linked do not match, or Z_ERRNO if there is
   an error reading or writing the files. */
int FROG_ZLIB_Deflate(FILE* source, FILE* dest, int level)
{
    int ret, flush;
    unsigned have;
    z_stream strm;
    unsigned char* in = new unsigned char [CHUNK];
    unsigned char* out= new unsigned char [CHUNK];

    /* allocate deflate state */
    strm.zalloc = Z_NULL;
    strm.zfree = Z_NULL;
    strm.opaque = Z_NULL;
//    ret = deflateInit(&strm, level);
    ret = deflateInit2(&strm, level, Z_DEFLATED, 31, 8, Z_DEFAULT_STRATEGY);
    if (ret != Z_OK)
        return ret;

    /* compress until end of file */
    do {
        strm.avail_in = fread(in, 1, CHUNK, source);
        if (ferror(source)) {
            (void)deflateEnd(&strm);
            return Z_ERRNO;
        }
        flush = feof(source) ? Z_FINISH : Z_NO_FLUSH;
        strm.next_in = in;

        /* run deflate() on input until output buffer not full, finish
           compression if all of source has been read in */
        do {
            strm.avail_out = CHUNK;
            strm.next_out = out;
            ret = deflate(&strm, flush);    /* no bad return value */
            assert(ret != Z_STREAM_ERROR);  /* state not clobbered */
            have = CHUNK - strm.avail_out;
            if (fwrite(out, 1, have, dest) != have || ferror(dest)) {
                (void)deflateEnd(&strm);
                return Z_ERRNO;
            }
        } while (strm.avail_out == 0);
        assert(strm.avail_in == 0);     /* all input will be used */

        /* done when last data in file processed */
    } while (flush != Z_FINISH);
    assert(ret == Z_STREAM_END);        /* stream will be complete */

    /* clean up and return */
    (void)deflateEnd(&strm);
    return Z_OK;
}




bool FROG_ZLIB_Inflate(char* source_path, bool keepIntermediateFile){
	unsigned int N = strlen(source_path);

	if(!FROG_ZLIB_IsGZip(source_path))return false;


	char* dest_path = new char[N];	
	strncpy(dest_path,source_path,N-3);
	dest_path[N-3] = '\0';

	// Check the dest file do not already exist!
	FILE* dest_stream = fopen(dest_path,"r");
	if(dest_stream){
		strcpy(source_path,dest_path);
		fclose(dest_stream);
		return true;
	}

	printf("Start Inflation of %s\n",source_path);

	FILE* src_stream  = fopen(source_path,"rb");
	if(!src_stream){return false;}

	dest_stream = fopen(dest_path,"wb");
	if(!dest_stream){return false;}

	int returnedValue =  FROG_ZLIB_Inflate(src_stream, dest_stream);
	fclose(src_stream);
	fclose(dest_stream);

	if(returnedValue==Z_OK){
		printf("File %s has been inflate corretly\n",source_path);
		printf("new name is %s\n",dest_path);		
		if(!keepIntermediateFile && remove(source_path)!=0)perror( "Error deleting file" );
		strcpy(source_path,dest_path);
		return true;
	}else if(returnedValue==Z_MEM_ERROR){
		printf("### ERROR ### Not Enough Memory to inflate the file : %s\n",source_path);
	}else if(returnedValue==Z_STREAM_ERROR){
		printf("### ERROR ### Stream error during the inflation of the file : %s\n",source_path);
	}else if(returnedValue==Z_VERSION_ERROR){
		printf("### ERROR ### Version error during the inflation of the file : %s\n",source_path);
	}else if(returnedValue==Z_ERRNO){
		printf("### ERROR ### Reading/Writing error during the inflation of the file : %s\n",source_path);
	}else{
		printf("%i\n",returnedValue);
	}

	return false;	
}

/* Decompress from file source to file dest until stream ends or EOF.
   inf() returns Z_OK on success, Z_MEM_ERROR if memory could not be
   allocated for processing, Z_DATA_ERROR if the deflate data is
   invalid or incomplete, Z_VERSION_ERROR if the version of zlib.h and
   the version of the library linked do not match, or Z_ERRNO if there
   is an error reading or writing the files. */
int FROG_ZLIB_Inflate(FILE* source, FILE* dest)
{
    int ret;
    unsigned have;
    z_stream strm;
    unsigned char* in = new unsigned char[CHUNK];
    unsigned char* out= new unsigned char[CHUNK];

    /* allocate inflate state */
    strm.zalloc = Z_NULL;
    strm.zfree = Z_NULL;
    strm.opaque = Z_NULL;
    strm.avail_in = 0;
    strm.next_in = Z_NULL;
    //ret = inflateInit(&strm);
	ret = inflateInit2(&strm,31);
    if (ret != Z_OK)
        return ret;

    /* decompress until deflate stream ends or end of file */
    do {
        strm.avail_in = fread(in, 1, CHUNK, source);
        if (ferror(source)) {
            (void)inflateEnd(&strm);
            return Z_ERRNO;
        }
        if (strm.avail_in == 0)
            break;
        strm.next_in = in;

        /* run inflate() on input until output buffer not full */
        do {
            strm.avail_out = CHUNK;
            strm.next_out = out;
            ret = inflate(&strm, Z_NO_FLUSH);
            assert(ret != Z_STREAM_ERROR);  /* state not clobbered */
            switch (ret) {
            case Z_NEED_DICT:
                ret = Z_DATA_ERROR;     /* and fall through */
            case Z_DATA_ERROR:
            case Z_MEM_ERROR:
                (void)inflateEnd(&strm);
                return ret;
            }
            have = CHUNK - strm.avail_out;
            if (fwrite(out, 1, have, dest) != have || ferror(dest)) {
                (void)inflateEnd(&strm);
                return Z_ERRNO;
            }
        } while (strm.avail_out == 0);

        /* done when inflate() says it's done */
    } while (ret != Z_STREAM_END);

    /* clean up and return */
    (void)inflateEnd(&strm);
    return ret == Z_STREAM_END ? Z_OK : Z_DATA_ERROR;
}
